/*
 * Soubor:  dynmstack.c
 * Datum:   17.03.2011 21:34
 * Autor:   Marek Nožka, nozka <@t> spseol <d.t> cz
 * Licence: GNU/GPL 
 * Úloha:   implementace zásobníku pomocí jednosměrného 
 *          lineárního seznamu
 */

#define _ISOC99_SOURCE
#define _GNU_SOURCE
/***********************************
 *   Hlavičkové soubory
 ***********************************/
#include "dynmstack.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/*********************************** 
 *  Globální proměnné a datové struktury
 ***********************************/
typedef struct stackbox {
    double number;
    struct stackbox *next;
} STACKBOX;

static STACKBOX *stack = NULL;


/*********************************** 
 *  Implementace funkcí
 ***********************************/

/* Funkce ukládá číslo do zásobníku,
 * vrací aktuální pozici nebo -1 při . 
 */
int push(double number)         //%.h   
{
    unsigned int size = 0;
    STACKBOX *newbox;

    if ((newbox = (STACKBOX *) malloc(sizeof(STACKBOX))) != NULL) {
        newbox->number = number;
        newbox->next = stack;
        stack = newbox;
        while (newbox != NULL) {
            newbox = newbox->next;
            size++;
        }
        return size;
    } else
        return -1;
}

/* Funkce vrací číslo, které vyzvedne ze zásobníku.
 * V případě prázdného zásobníku vrátí NAN. */
double pop()                    //%.h   
{
    STACKBOX *forfree;
    double ret;
    if (stack != NULL) {
        forfree = stack;
        stack = forfree->next;
        ret = forfree->number;
        free((void *) forfree);
        return ret;
    } else
        return NAN;
}

/* Funkce vrací počet prvků v zásobníku */
int gettstackc(void)            //%.h
{
    STACKBOX *aktual = stack;
    unsigned int size = 0;
    while (aktual != NULL) {
        aktual = aktual->next;
        size++;
    }
    return size;
}

/* Funkce do řetězce s vytiskne obsah zásobníku */
void spstack(char s[])          //%.h
{
    STACKBOX *aktual = stack;
    char bagr[MSGLEN];

    strncpy(s, "]", MSGLEN);
    while (aktual != NULL) {
        snprintf(bagr, MSGLEN, "%g ", aktual->number);
        strncat(bagr, s, MSGLEN);
        strncpy(s, bagr, MSGLEN);
        aktual = aktual->next;
    }
    strncpy(bagr, "[ ", MSGLEN);
    strncat(bagr, s, MSGLEN);
    strncpy(s, bagr, MSGLEN);
}

/* Vymaže celý zásobník */
void stackclear(void)           //%.h
{
    STACKBOX *forfree = stack;
    while (stack != NULL) {
        forfree = stack;
        stack = stack->next;
        free((void *) forfree);
    }
}

/*
int main()
{
    char s[MSGLEN];

    printf("   %d\n", push(8));
    printf("   %d\n", push(12));
    printf("   %d\n", push(13));
    printf("   %d\n", push(20));

    printf("     %d\n", gettstackc());

    spstack(s);
    printf("#### %s\n", s);

    printf("%g\n", pop());
    printf("%g\n", pop());
    printf("     %d\n", gettstackc());
    printf("%g\n", pop());
    printf("%g\n", pop());
    printf("%g\n", pop());

    stackclear();
}
*/
